@@ -34,7 +34,7 @@
 #include <utility>
 #include "ol-exceptions.h"
 
-namespace hfst_ol {
+namespace hfst_ospell {
 
 typedef uint16_t SymbolNumber;
 typedef uint32_t TransitionTableIndex;
@@ -246,6 +246,7 @@ public:
     //!
     //! find a key for string or add it
     SymbolNumber find_key(char ** p);
+    bool has_key_starting_with(const char c) const;
     ~LetterTrie();
 };
 
@@ -457,6 +458,6 @@ void debug_print(printable p)
     }
 }
 
-} // namespace hfst_ol
+} // namespace hfst_ospell
 
 #endif // HFST_OSPELL_HFST_OL_H_
