@@ -21,16 +21,16 @@
 */
 
 /*
-	Tests up to 16 variations of each input token:
+	Tests up to 8 variations of each input token:
 	- Verbatim
 	- With leading non-alphanumerics removed
 	- With trailing non-alphanumerics removed
 	- With leading and trailing non-alphanumerics removed
-	- Lower-case of all the above
-	- First-upper of all the above
+	- First-lower of all the above
 */
 
 #include <iostream>
+#include <iomanip>
 #include <fstream>
 #include <vector>
 #include <string>
@@ -42,6 +42,7 @@
 #include <cmath>
 #include <cerrno>
 #include <cctype>
+#include <getopt.h>
 
 #define U_CHARSET_IS_UTF8 1
 #include <unicode/uclean.h>
@@ -64,13 +65,18 @@ struct word_t {
 	UnicodeString buffer;
 };
 std::vector<word_t> words(16);
-std::string buffer;
-std::vector<std::string> alts;
+std::string buffer, wbuf;
+using Alt = std::pair<double,std::string>;
+std::vector<Alt> alts;
 std::unordered_set<std::string> outputs;
 UnicodeString ubuffer, uc_buffer;
 size_t cw;
 
 bool verbatim = false;
+bool debug = false;
+hfst_ospell::Weight max_weight = -1.0;
+hfst_ospell::Weight beam = -1.0;
+float time_cutoff = 6.0;
 bool uc_first = false;
 bool uc_all = true;
 
@@ -82,14 +88,18 @@ bool find_alternatives(ZHfstOspeller& speller, size_t suggs) {
 	for (size_t k=0 ; k < cw && alts.size()<suggs ; ++k) {
 		buffer.clear();
 		words[k].buffer.toUTF8String(buffer);
-		hfst_ospell::CorrectionQueue corrections = speller.suggest(buffer);
+		auto corrections = speller.suggest(buffer);
 
 		if (corrections.size() == 0) {
 			continue;
 		}
 
-		// Because speller.set_queue_limit() doesn't actually work, hard limit it here
-		for (size_t i=0, e=corrections.size() ; i<e && alts.size()<suggs ; ++i) {
+		for (size_t i=0, e=corrections.size() ; i<e ; ++i) {
+			// Work around https://github.com/hfst/hfst-ospell/issues/54
+			if (max_weight > 0.0 && corrections.top().second > max_weight) {
+				break;
+			}
+			auto w = corrections.top().second * (1.0 + k/10.0);
 
 			buffer.clear();
 			if (k != 0) {
@@ -112,8 +122,18 @@ bool find_alternatives(ZHfstOspeller& speller, size_t suggs) {
 				words[0].buffer.tempSubString(words[k].start + words[k].count).toUTF8String(buffer);
 			}
 
+			if (debug) {
+				wbuf.resize(64);
+				wbuf.resize(sprintf(&wbuf[0], " (%.2f;%zu)", corrections.top().second, k));
+				buffer += wbuf;
+			}
+
 			if (outputs.count(buffer) == 0) {
-				alts.push_back(buffer);
+				alts.push_back({w, buffer});
+				std::sort(alts.begin(), alts.end());
+				while (alts.size() > suggs) {
+					alts.pop_back();
+				}
 			}
 			outputs.insert(buffer);
 			corrections.pop();
@@ -123,7 +143,7 @@ bool find_alternatives(ZHfstOspeller& speller, size_t suggs) {
 	if (!alts.empty()) {
 		std::cout << "&";
 		for (auto& alt : alts) {
-			std::cout << "\t" << alt;
+			std::cout << "\t" << alt.second;
 		}
 		std::cout << std::endl;
 		return true;
@@ -167,7 +187,7 @@ bool is_valid_word(ZHfstOspeller& speller, const std::string& word, size_t suggs
 	}
 
 	size_t ichStart = 0, cchUse = ubuffer.length();
-	const UChar *pwsz = ubuffer.getTerminatedBuffer();
+	auto pwsz = ubuffer.getTerminatedBuffer();
 
 	// Always test the full given input
 	words[0].buffer.remove();
@@ -216,7 +236,7 @@ bool is_valid_word(ZHfstOspeller& speller, const std::string& word, size_t suggs
 
 	for (size_t i=0, e=cw ; i<e ; ++i) {
 		// If we are looking for suggestions, don't use the cache
-		valid_words_t::iterator it = suggs ? valid_words.end() : valid_words.find(words[i].buffer);
+		auto it = suggs ? valid_words.end() : valid_words.find(words[i].buffer);
 
 		if (it == valid_words.end()) {
 			buffer.clear();
@@ -224,49 +244,21 @@ bool is_valid_word(ZHfstOspeller& speller, const std::string& word, size_t suggs
 			bool valid = speller.spell(buffer);
 			it = valid_words.insert(std::make_pair(words[i].buffer,valid)).first;
 
-			if (!valid && !verbatim) {
-				// If the word was not valid, fold it to lower case and try again
-				buffer.clear();
-				ubuffer = words[i].buffer;
-				ubuffer.toLower();
-				ubuffer.toUTF8String(buffer);
-
-				// Add the lower case variant to the list so that we get suggestions using that, if need be
-				words[cw].start = words[i].start;
-				words[cw].count = words[i].count;
-				words[cw].buffer = ubuffer;
-				++cw;
-
-				// Don't try again if the lower cased variant has already been tried
-				valid_words_t::iterator itl = suggs ? valid_words.end() : valid_words.find(ubuffer);
-				if (itl != valid_words.end()) {
-					it->second = itl->second;
-					it = itl;
-				}
-				else {
-					valid = speller.spell(buffer);
-					it->second = valid; // Also mark the original mixed case variant as whatever the lower cased one was
-					it = valid_words.insert(std::make_pair(words[i].buffer,valid)).first;
-				}
-			}
-
-			if (!valid && !verbatim && (uc_all || uc_first)) {
-				// If the word was still not valid but had upper case, try a first-upper variant
+			if (!valid && !verbatim && uc_first) {
+				// If the word was not valid, try a first-lower variant
 				buffer.clear();
 				ubuffer.setTo(words[i].buffer, 0, 1);
-				ubuffer.toUpper();
-				uc_buffer.setTo(words[i].buffer, 1);
-				uc_buffer.toLower();
-				ubuffer.append(uc_buffer);
+				ubuffer.toLower();
+				ubuffer.append(words[i].buffer, 1, words[i].buffer.length() - 1);
 				ubuffer.toUTF8String(buffer);
 
-				// Add the first-upper variant to the list so that we get suggestions using that, if need be
+				// Add the first-lower case variant to the list so that we get suggestions using that, if need be
 				words[cw].start = words[i].start;
 				words[cw].count = words[i].count;
 				words[cw].buffer = ubuffer;
 				++cw;
 
-				// Don't try again if the first-upper variant has already been tried
+				// Don't try again if the first-lower variant has already been tried
 				valid_words_t::iterator itl = suggs ? valid_words.end() : valid_words.find(ubuffer);
 				if (itl != valid_words.end()) {
 					it->second = itl->second;
@@ -274,7 +266,7 @@ bool is_valid_word(ZHfstOspeller& speller, const std::string& word, size_t suggs
 				}
 				else {
 					valid = speller.spell(buffer);
-					it->second = valid; // Also mark the original mixed case variant as whatever the first-upper one was
+					it->second = valid; // Also mark the original mixed case variant as whatever the first-lower one was
 					it = valid_words.insert(std::make_pair(words[i].buffer,valid)).first;
 				}
 			}
@@ -291,8 +283,13 @@ bool is_valid_word(ZHfstOspeller& speller, const std::string& word, size_t suggs
 int zhfst_spell(const char* zhfst_filename) {
 	ZHfstOspeller speller;
 	try {
+		if (debug) {
+			std::cout << "@@ Loading " << zhfst_filename << " with args max-weight=" << max_weight << ", beam=" << beam << ", time-cutoff=" << time_cutoff << std::endl;
+		}
 		speller.read_zhfst(zhfst_filename);
-		speller.set_time_cutoff(6.0);
+		speller.set_weight_limit(max_weight);
+		speller.set_beam(beam);
+		speller.set_time_cutoff(time_cutoff);
 	}
 	catch (hfst_ospell::ZHfstMetaDataParsingError zhmdpe) {
 		fprintf(stderr, "cannot finish reading zhfst archive %s:\n%s.\n", zhfst_filename, zhmdpe.what());
@@ -319,6 +316,38 @@ int zhfst_spell(const char* zhfst_filename) {
 		if (line.empty()) {
 			continue;
 		}
+
+		if (line.size() >= 5 && line[0] == '$' && line[1] == '$' && line[3] == ' ') {
+			if (line[2] == 'd' && isdigit(line[4]) && line.size() == 5) {
+				debug = (line[4] != '0');
+				std::cout << "@@ Option debug changed to " << debug << std::endl;
+				continue;
+			}
+			if (line[2] == 'T' && isdigit(line[4]) && line.size() == 5) {
+				verbatim = (line[4] != '0');
+				std::cout << "@@ Option verbatim changed to " << verbatim << std::endl;
+				continue;
+			}
+			if (line[2] == 'w' && isdigit(line[4])) {
+				max_weight = std::stof(&line[4]);
+				speller.set_weight_limit(max_weight);
+				std::cout << "@@ Option max-weight changed to " << max_weight << std::endl;
+				continue;
+			}
+			if (line[2] == 'b' && isdigit(line[4])) {
+				beam = std::stof(&line[4]);
+				speller.set_beam(beam);
+				std::cout << "@@ Option beam changed to " << beam << std::endl;
+				continue;
+			}
+			if (line[2] == 't' && isdigit(line[4])) {
+				time_cutoff = std::stof(&line[4]);
+				speller.set_time_cutoff(time_cutoff);
+				std::cout << "@@ Option time-cutoff changed to " << time_cutoff << std::endl;
+				continue;
+			}
+		}
+
 		// Just in case anyone decides to use the speller for a minor eternity
 		if (valid_words.size() > 20480) {
 			valid_words.clear();
@@ -345,6 +374,19 @@ int zhfst_spell(const char* zhfst_filename) {
     return EXIT_SUCCESS;
 }
 
+void print_help() {
+	std::cout
+		<< "Usage: hfst-ospell [options] zhfst-archive\n"
+		<< "\n"
+		<< " -h, --help            Shows this help\n"
+		<< " -d, --debug           Debug output with weights attached to results\n"
+		<< " -T, --verbatim        Disables case-folding and non-alphanumeric trimming\n"
+		<< " -w, --max-weight=W    Suppress corrections with weights above W\n"
+		<< " -b, --beam=W          Suppress corrections worse than best candidate by more than W\n"
+		<< " -t, --time-cutoff=T   Stop trying to find better corrections after T seconds; defaults to 6.0\n"
+		<< std::flush;
+}
+
 int main(int argc, char **argv) {
 	UErrorCode status = U_ZERO_ERROR;
 	u_init(&status);
@@ -356,22 +398,60 @@ int main(int argc, char **argv) {
 	ucnv_setDefaultName("UTF-8");
 	uloc_setDefault("en_US_POSIX", &status);
 
-	std::vector<std::string> args(argv, argv+argc);
-	for (std::vector<std::string>::iterator it=args.begin() ; it != args.end() ; ) {
-		if (*it == "--verbatim") {
-			verbatim = true;
-			it = args.erase(it);
+	struct option long_options[] =
+		{
+		{"help",         no_argument,       0, 'h'},
+		{"debug",        no_argument,       0, 'd'},
+		{"verbatim",     no_argument,       0, 'T'},
+		{"max-weight",   required_argument, 0, 'w'},
+		{"beam",         required_argument, 0, 'b'},
+		{"time-cutoff",  required_argument, 0, 't'},
+		{0,              0,                 0,  0 }
+		};
+
+	int c = 0;
+	while (true) {
+		int option_index = 0;
+		c = getopt_long(argc, argv, "hdTw:b:t:", long_options, &option_index);
+
+		if (c == -1) {
+			break;
 		}
-		else {
-			++it;
+
+		switch (c) {
+		case 'h':
+			print_help();
+			return EXIT_SUCCESS;
+
+		case 'd':
+			debug = true;
+			break;
+
+		case 'T':
+			verbatim = true;
+			break;
+
+		case 'w':
+			max_weight = std::stof(optarg);
+			break;
+
+		case 'b':
+			beam = std::stof(optarg);
+			break;
+
+		case 't':
+			time_cutoff = std::stof(optarg);
+			break;
 		}
 	}
 
-	if (args.size() < 2) {
+	if (optind >= argc) {
 		throw std::invalid_argument("Must pass a zhfst as argument");
 	}
 
-	int rv = zhfst_spell(args[1].c_str());
+	std::cerr << std::fixed << std::setprecision(2);
+	std::cout << std::fixed << std::setprecision(2);
+	int rv = zhfst_spell(argv[optind]);
 
 	u_cleanup();
 	return rv;
